# GrantUsersPrivilegesOnSelectedFolders_Server.ps1 - Grants selected users privileges on other archives
#                                                    The privileges can be selected
#                                                    Certain archives can be excluded
#
# Copyright (c) 2017 - 2024 MailStore Software GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

# https://help.mailstore.com/en/server/PowerShell_API_Wrapper_Tutorial

# API Wrapper in same directory is used, nothing to configure here
Import-Module (Join-Path $PSScriptRoot "MS.PS.Lib.psd1") -ErrorAction Stop

######################################################
# Adjust these values according to your installation #
######################################################

# Username of an MailStore administrator
# default is "admin"
$username = "admin"

# If password is empty, an interactive prompt will ask for it
$password = ''

# The hostname of the MailStore Server
# default is "localhost"
$hostname = "localhost"

# TCP Port of the Management API
# API must be enabled in the MailStore Server Service Configuration
# default is 8463
$port = 8463

# Set this to $true when untrusted certificates are used
# default is $false
$acceptAllCertificates = $false

# Which users should receive the privileges
# comma-separated list of users
$users = "user001", "user002"

# Which privileges should they receive
# default = "read"
# read+write = "read,write"
# read+delete = "read,delete"
# read+write+delete = "read,write,delete"
# removeall = "none"
$privilege = "read"

# Which archives are to exclude
# comma-separated list of archives
# leave blank, to not exclude any archives
# $excluded_archives =
$excluded_archives = "admin", "@catchall"


#### DO NOT MAKE ANY MODIFICATIONS BELOW THIS LINE ####

if ($password -ceq ''){
    $password = [System.Runtime.InteropServices.Marshal]::PtrToStringAuto([System.Runtime.InteropServices.Marshal]::SecureStringToBSTR((Read-Host -AsSecureString -Prompt "Enter MailStore Password for $username")))
}

$client = New-MSApiClient -Username $username -Password $password -MailStoreServer $hostname -Port $port -IgnoreInvalidSSLCerts:$acceptAllCertificates
$archivesresult = (Invoke-MSApiCall $client "GetChildFolders" @{"maxLevels"=2})

if ($archivesresult.statusCode -ne "succeeded"){
    Write-Host ("{0}" -f $archivesresult.error)
    exit
}

foreach ($user in $users){
    foreach ($archive in $archivesresult.result.childFolders.name){
        if (-not( $excluded_archives -contains $archive)){
            $result = (Invoke-MSApiCall $client "SetUserPrivilegesOnFolder" @{"userName"=$user; "folder"=$archive; "privileges"=$privilege})
            $success = $result.statusCode -eq "succeeded"
            Write-Host ("{0} -> {1} ({2}): {3}" -f $user, $archive, $privilege, $success)
            if (-not $success){
                Write-Host ("{0}" -f $result.error)
            }
        }
    }
}

Write-Host ""
Write-Host "Script finished"